/*
 Sasl 1.x python module
 Copyright (C) 2001 Gianluigi Tiesi <sherpya@netfarm.it>
 Copyright (C) 2001 NetFarm S.r.l.  [http://www.netfarm.it]

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 2, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
*/


#define SASLMOD_VERSION "0.3"
#define SASLMOD_DOC "Cyrus sasl module for user administration"
#include <Python.h>
#include <sasl.h>

/* Proto */
static PyObject *Sasl_deleteuser (PyObject * self, PyObject * args);
static PyObject *Sasl_createuser (PyObject * self, PyObject * args);
static PyObject *Sasl_setpass (PyObject * self, PyObject * args);
static PyObject *Error;
void initSasl(void);

/* Method list */
static PyMethodDef Sasl_methods[] = {
  {"deleteuser", Sasl_deleteuser, METH_VARARGS},
  {"createuser", Sasl_createuser, METH_VARARGS},
  {"setpass", Sasl_setpass, METH_VARARGS},
  {NULL, NULL}
};

/* Init Function */
void
initSasl ()
{
  PyObject *dict, *module;

  module = Py_InitModule ("Sasl", Sasl_methods);
  dict = PyModule_GetDict (module);

  PyDict_SetItemString (dict, "__version__",
			PyString_FromString (SASLMOD_VERSION));
  PyDict_SetItemString (dict, "__doc__", PyString_FromString (SASLMOD_DOC));
  Error = PyErr_NewException ("Saslmodule.Error", NULL, NULL);
  PyDict_SetItemString (dict, "error", Error);

  /* Check for errors in init */
  if (PyErr_Occurred ())
    Py_FatalError ("Can't initialize Saslmodule");

};

sasl_conn_t *
_initsasl (void)
{
  sasl_conn_t *conn;
  if ((sasl_server_init (NULL, "saslpasswd") == SASL_OK) &&
      (sasl_server_new ("saslpasswd", NULL, NULL, NULL, 0, &conn) == SASL_OK))
    return conn;
  else
    return NULL;
}


/* Sasl_deleteuser */
static PyObject *
Sasl_deleteuser (PyObject * self, PyObject * args)
{
  PyObject *obj_result;
  unsigned char *user;
  const char *errstr = NULL;
  int result = 0;
  sasl_conn_t *conn;

  if (!PyArg_ParseTuple (args, "s", &user))
    return NULL;

  conn = _initsasl ();
  if (conn == NULL)
    {
      PyErr_SetString (Error, "Saslmodule: error opening sasldb");
      return NULL;
    }

  result = sasl_setpass (conn, user, NULL, 0, SASL_SET_DISABLE, &errstr);

  if (result != SASL_OK)
    {
      PyErr_SetString (Error, sasl_errstring (result, NULL, NULL));
      return NULL;
    }

  obj_result = Py_BuildValue ("i", result);

  return obj_result;
}

/* Sasl_createuser */
static PyObject *
Sasl_createuser (PyObject * self, PyObject * args)
{
  PyObject *obj_result;
  unsigned char *user, *pass;
  const char *errstr = NULL;
  sasl_conn_t *conn;
  int result;
  int passlen = 0;

  if (!PyArg_ParseTuple (args, "ss", &user, &pass))
    return NULL;

  conn = _initsasl ();
  if (conn == NULL)
    {
      PyErr_SetString (Error, "Saslmodule: error opening sasldb");
      return NULL;
    }

  passlen = strlen (pass);
  result = sasl_setpass (conn, user, pass, passlen, SASL_SET_CREATE, &errstr);

  if (result != SASL_OK)
    {
      PyErr_SetString (Error, sasl_errstring (result, NULL, NULL));
      return NULL;
    }

  obj_result = Py_BuildValue ("i", result);

  return obj_result;

}

/* Sasl_setpass */
static PyObject *
Sasl_setpass (PyObject * self, PyObject * args)
{
  PyObject *obj_result;
  unsigned char *user, *pass;
  const char *errstr = NULL;
  sasl_conn_t *conn;
  int result;
  int passlen = 0;

  if (!PyArg_ParseTuple (args, "ss", &user, &pass))
    return NULL;

  conn = _initsasl ();
  if (conn == NULL)
    {
      PyErr_SetString (Error, "Saslmodule: error opening sasldb");
      return NULL;
    }

  passlen = strlen (pass);
  result = sasl_setpass (conn, user, pass, passlen, 0, &errstr);

  if (result != SASL_OK)
    {
      PyErr_SetString (Error, sasl_errstring (result, NULL, NULL));
      return NULL;
    }

  obj_result = Py_BuildValue ("i", result);

  return obj_result;

}
